@@ -46,6 +46,7 @@ struct _GGduDrive {
   GIcon *icon;
   gchar *name;
   gchar *device_file;
+  dev_t dev;
   gboolean is_media_removable;
   gboolean has_media;
   gboolean can_eject;
@@ -128,6 +129,7 @@ update_drive (GGduDrive *drive)
   GIcon *old_icon;
   gchar *old_name;
   gchar *old_device_file;
+  dev_t old_dev;
   gboolean old_is_media_removable;
   gboolean old_has_media;
   gboolean old_can_eject;
@@ -151,6 +153,7 @@ update_drive (GGduDrive *drive)
 
   old_name = g_strdup (drive->name);
   old_device_file = g_strdup (drive->device_file);
+  old_dev = drive->dev;
   old_icon = drive->icon != NULL ? g_object_ref (drive->icon) : NULL;
 
   /* in with the new */
@@ -166,12 +169,15 @@ update_drive (GGduDrive *drive)
   else
     drive->name = gdu_presentable_get_name (drive->presentable);
 
-  /* the GduDevice for an activatable drive (such as RAID) is NULL if the drive is not activated */
+  /* It's perfectly fine to not have a GduDevice - for example, this is the case for non-running
+   * MD RAID arrays as well as LVM2 Volume Group "drives"
+   */
   if (device == NULL)
     {
       g_free (drive->device_file);
+      drive->dev = 0;
       drive->device_file = NULL;
-      drive->is_media_removable = TRUE;
+      drive->is_media_removable = FALSE;
       drive->has_media = TRUE;
       drive->can_eject = FALSE;
       drive->can_poll_for_media = FALSE;
@@ -179,6 +185,7 @@ update_drive (GGduDrive *drive)
   else
     {
       g_free (drive->device_file);
+      drive->dev = gdu_device_get_dev (device);
       drive->device_file = g_strdup (gdu_device_get_device_file (device));
       drive->is_media_removable = gdu_device_is_removable (device);
       drive->has_media = gdu_device_is_media_available (device);
@@ -265,6 +272,7 @@ update_drive (GGduDrive *drive)
               (old_can_poll_for_media == drive->can_poll_for_media) &&
               (g_strcmp0 (old_name, drive->name) == 0) &&
               (g_strcmp0 (old_device_file, drive->device_file) == 0) &&
+              (old_dev == drive->dev) &&
               g_icon_equal (old_icon, drive->icon)
               );
 
@@ -810,45 +818,44 @@ g_gdu_drive_stop_on_all_unmounted (GDrive              *_drive,
   GSimpleAsyncResult *simple;
   GduDevice *device;
 
-  device = gdu_presentable_get_device (drive->presentable);
-  if (device == NULL)
-    {
-      simple = g_simple_async_result_new_error (G_OBJECT (drive),
-                                                callback,
-                                                user_data,
-                                                G_IO_ERROR,
-                                                G_IO_ERROR_FAILED,
-                                                "Drive is activatable and not running");
-      g_simple_async_result_complete_in_idle (simple);
-      g_object_unref (simple);
-    }
-  else
-    {
-      simple = g_simple_async_result_new (G_OBJECT (drive),
-                                          callback,
-                                          user_data,
-                                          NULL);
+  simple = g_simple_async_result_new (G_OBJECT (drive),
+                                      callback,
+                                      user_data,
+                                      NULL);
 
-      switch (drive->start_stop_type)
+  switch (drive->start_stop_type)
+    {
+    case G_DRIVE_START_STOP_TYPE_SHUTDOWN:
+      device = gdu_presentable_get_device (drive->presentable);
+      if (device == NULL)
         {
-        case G_DRIVE_START_STOP_TYPE_SHUTDOWN:
-          gdu_device_op_drive_detach (device, stop_cb, simple);
-          break;
-
-        case G_DRIVE_START_STOP_TYPE_MULTIDISK:
-          gdu_drive_deactivate (GDU_DRIVE (drive->presentable), drive_deactivate_cb, simple);
-          break;
-
-        default:
           g_simple_async_result_set_error (simple,
                                            G_IO_ERROR,
-                                           G_IO_ERROR_NOT_SUPPORTED,
-                                           "start_stop_type %d not supported",
-                                           drive->start_stop_type);
+                                           G_IO_ERROR_FAILED,
+                                           "Cannot detach: drive has no GduDevice object");
           g_simple_async_result_complete_in_idle (simple);
           g_object_unref (simple);
-          break;
         }
+      else
+        {
+          gdu_device_op_drive_detach (device, stop_cb, simple);
+          g_object_unref (device);
+        }
+      break;
+
+    case G_DRIVE_START_STOP_TYPE_MULTIDISK:
+      gdu_drive_deactivate (GDU_DRIVE (drive->presentable), drive_deactivate_cb, simple);
+      break;
+
+    default:
+      g_simple_async_result_set_error (simple,
+                                       G_IO_ERROR,
+                                       G_IO_ERROR_NOT_SUPPORTED,
+                                       "start_stop_type %d not supported",
+                                       drive->start_stop_type);
+      g_simple_async_result_complete_in_idle (simple);
+      g_object_unref (simple);
+      break;
     }
 }
 
@@ -1200,17 +1207,17 @@ g_gdu_drive_drive_iface_init (GDriveIface *iface)
 }
 
 gboolean
-g_gdu_drive_has_device_file (GGduDrive      *drive,
-                             const gchar    *device_file)
+g_gdu_drive_has_dev (GGduDrive      *drive,
+                     dev_t           dev)
 {
-  return g_strcmp0 (drive->device_file, device_file) == 0;
+  return drive->dev == dev;
 }
 
 gboolean
 g_gdu_drive_has_presentable (GGduDrive       *drive,
                              GduPresentable  *presentable)
 {
-  return gdu_presentable_get_id (drive->presentable) == gdu_presentable_get_id (presentable);
+  return g_strcmp0 (gdu_presentable_get_id (drive->presentable), gdu_presentable_get_id (presentable)) == 0;
 }
 
 time_t
