@@ -105,6 +105,7 @@ main (int argc, char *argv[])
   char *basename;
   int i;
   GFileCopyFlags flags;
+  int retval = 0;
 
   setlocale (LC_ALL, "");
 
@@ -205,6 +206,7 @@ main (int argc, char *argv[])
 	    copy_failed:
 	      g_printerr (_("Error copying file %s: %s\n"), argv[i], error->message);
 	      g_error_free (error);
+	      retval = 1;
 	    }
 	}
 
@@ -214,5 +216,5 @@ main (int argc, char *argv[])
 
   g_object_unref (dest);
 
-  return 0;
+  return retval;
 }
